import { NodeProp } from '@lezer/common';
import { parser } from 'lezer-julia';
import { LRLanguage, indentNodeProp, continuedIndent, LanguageSupport } from '@codemirror/language';
import * as autocomplete from '@codemirror/autocomplete';
import * as highlight from '@codemirror/highlight';

function delimitedStrategy(context, units, closing) {
    let after = context.textAfter;
    let space = after.match(/^\s*/)[0].length;
    let closed = false;
    switch (closing.length) {
        case 1:
            closed = after.slice(space, space + closing[0].length) === closing[0];
            break;
        case 2:
            closed =
                after.slice(space, space + closing[0].length) === closing[0] ||
                    after.slice(space, space + closing[1].length) === closing[1];
            break;
        case 3:
            closed =
                after.slice(space, space + closing[0].length) === closing[0] ||
                    after.slice(space, space + closing[1].length) === closing[1] ||
                    after.slice(space, space + closing[2].length) === closing[2];
            break;
        default:
            closed = closing.some((closing) => after.slice(space, space + closing.length) === closing);
            break;
    }
    return context.baseIndent + (closed ? 0 : context.unit * units);
}
function delimitedIndent({ closing, units = 1, }) {
    return (context) => delimitedStrategy(context, units, closing);
}
function noIndent(context) {
    return context.baseIndent;
}

function getSyntaxConfig() {
    let syntaxConfig = {
        indents: {
            VariableDeclaration: continuedIndent(),
            AssignmentExpression: continuedIndent(),
        },
        keywords: [],
    };
    for (let node of parser.nodeSet.types) {
        // Collect keywords
        let groups = node.prop(NodeProp.group);
        let group = groups != null ? groups[0] : null;
        if (group === "keyword") {
            syntaxConfig.keywords.push(node);
        }
        // Configure indents
        let nodeIndent;
        let closedBy = node.prop(NodeProp.closedBy);
        if (closedBy) {
            nodeIndent = delimitedIndent({ closing: closedBy });
        }
        else {
            nodeIndent = noIndent;
        }
        syntaxConfig.indents[node.name] = nodeIndent;
    }
    return syntaxConfig;
}
let syntaxConfig = /*@__PURE__*/getSyntaxConfig();
let styleTags = /*@__PURE__*/highlight.styleTags({
    Identifier: highlight.tags.variableName,
    String: highlight.tags.string,
    TripleString: highlight.tags.string,
    Comment: highlight.tags.lineComment,
    BlockComment: highlight.tags.comment,
    [/*@__PURE__*/syntaxConfig.keywords.map((t) => t.name).join(" ")]: highlight.tags.keyword,
    "( )": highlight.tags.paren,
    "[ ]": highlight.tags.paren,
    "{ }": highlight.tags.paren,
    MacroIdentifier: highlight.tags.macroName,
});
let language = /*@__PURE__*/LRLanguage.define({
    parser: /*@__PURE__*/parser.configure({
        props: [
            styleTags,
            /*@__PURE__*/indentNodeProp.add(/*@__PURE__*/Object.assign(/*@__PURE__*/Object.assign({}, syntaxConfig.indents), { ModuleDefinition: noIndent, BareModuleDefinition: noIndent, VariableDeclaration: /*@__PURE__*/continuedIndent(), AssignmentExpression: /*@__PURE__*/continuedIndent() })),
        ],
    }),
    languageData: {
        commentTokens: { line: "#" },
        indentOnInput: /^\s*(\]|\}|\)|end|else|elseif|catch|finally)/,
        closeBrackets: { brackets: ["(", "[", "{", "'", '"', "`"] },
    },
});
const keywordCompletion = /*@__PURE__*/language.data.of({
    autocomplete: /*@__PURE__*/autocomplete.completeFromList(/*@__PURE__*/syntaxConfig.keywords.map((keyword) => ({
        label: keyword.name,
        type: "keyword",
    }))),
});
let defaultConfig = {
    enableKeywordCompletion: false,
};
function julia(config = defaultConfig) {
    config = Object.assign(Object.assign({}, defaultConfig), config);
    let extensions = [];
    if (config.enableKeywordCompletion) {
        extensions.push(keywordCompletion);
    }
    return new LanguageSupport(language, extensions);
}

export { julia, keywordCompletion };
